from .exceptions import JenkinsError as JenkinsError
from typing import Any

class Views:
    jenkins: Any = ...
    def __init__(self, jenkins: Any) -> None: ...
    async def get_all(self) -> dict: ...
    async def is_exists(self, name: str) -> bool: ...
    async def get_config(self, name: str) -> str: ...
    async def create(self, name: str, config: str) -> None: ...
    async def reconfigure(self, name: str, config: str) -> None: ...
    async def delete(self, name: str) -> None: ...



from .exceptions import JenkinsNotFoundError as JenkinsNotFoundError
from .utils import parse_build_url as parse_build_url
from typing import Any, Optional, Tuple

class Builds:
    jenkins: Any = ...
    def __init__(self, jenkins: Any) -> None: ...
    @staticmethod
    def parse_url(build_url: str) -> Tuple[str, int]: ...
    async def get_all(self, name: str) -> list: ...
    async def get_info(self, name: str, build_id: int) -> dict: ...
    async def get_url_info(self, build_url: str) -> dict: ...
    async def get_output(self, name: str, build_id: int) -> str: ...
    async def is_exists(self, name: str, build_id: int) -> bool: ...
    async def get_queue_id_info(self, queue_id: int) -> dict: ...
    async def start(self, name: str, parameters: Optional[dict]=..., delay: int=...) -> Optional[int]: ...
    async def stop(self, name: str, build_id: int) -> None: ...
    async def delete(self, name: str, build_id: int) -> None: ...



from .exceptions import JenkinsError


class Views:

    def __init__(self, jenkins):
        self.jenkins = jenkins

    async def get_all(self) -> dict:
        status = await self.jenkins.get_status()
        return {v['name']: v for v in status['views']}

    async def is_exists(self, name: str) -> bool:
        views = await self.get_all()
        return name in views

    async def get_config(self, name: str) -> str:
        """
        Return view config in XML format
        """
        response = await self.jenkins._request(
            'GET',
            '/view/{}/config.xml'.format(name)
        )

        return await response.text()

    async def create(self, name: str, config: str) -> None:
        if name in await self.get_all():
            raise JenkinsError('View `{}` is already exists'.format(name))

        headers = {'Content-Type': 'text/xml'}
        params = {'name': name}

        await self.jenkins._request(
            'POST',
            '/createView',
            data=config,
            params=params,
            headers=headers,
        )

    async def reconfigure(self, name: str, config: str) -> None:
        await self.jenkins._request(
            'POST',
            '/view/{}/config.xml'.format(name),
            data=config,
            headers={'Content-Type': 'text/xml'},
        )

    async def delete(self, name: str) -> None:
        await self.jenkins._request('POST', '/view/{}/doDelete'.format(name))



import json
import xml.etree.ElementTree

from typing import Any, List

from .exceptions import JenkinsError, JenkinsNotFoundError
from .utils import construct_node_config, parse_build_url


def _parse_rss(rss: str) -> list:
    builds = []
    ns = {'atom': 'http://www.w3.org/2005/Atom'}

    root = xml.etree.ElementTree.fromstring(rss)
    for entry in root.findall('atom:entry', ns):
        link = entry.find('atom:link', ns)
        if link is not None:
            build_url = link.attrib['href']
            job_name, build_id = parse_build_url(build_url)

            builds.append({
                'url': build_url,
                'job_name': job_name,
                'number': build_id,
            })

    return list(reversed(builds))


class Nodes:

    def __init__(self, jenkins):
        self.jenkins = jenkins

    @staticmethod
    def _normalize_name(name: str) -> str:
        # embedded node `master` actually have brackets in HTTP requests
        if name == 'master':
            return '(master)'
        return name

    async def get_all(self) -> dict:
        """
        Get available nodes on server. Example dict result:
        {
            "master": dict(...),
            "buildbot1": dict(...)
        }
        """
        response = await self.jenkins._request(
            'GET',
            '/computer/api/json'
        )

        nodes = await response.json()
        return {v['displayName']: v for v in nodes['computer']}

    async def get_info(self, name: str) -> dict:
        name = self._normalize_name(name)
        response = await self.jenkins._request(
            'GET',
            '/computer/{}/api/json'.format(name),
        )
        return await response.json()

    async def get_failed_builds(self, name: str) -> List[dict]:
        """
        Return list of detalizied failed builds for node name, actually it
        parsed from RSS feed. usefull for build restart. Ascending builds sort.

        Example: [{
          'job_name': 'test',
          'number': 1,
          'url': 'http://localhost:8080/job/test/1/'
        }]
        """
        name = self._normalize_name(name)
        response = await self.jenkins._request(
            'GET',
            '/computer/{}/rssFailed'.format(name),
        )
        return _parse_rss(await response.text())

    async def get_all_builds(self, name: str) -> List[dict]:
        """
        Return list of all detalizied builds for node name, actually it parsed
        from RSS feed. Ascending builds sort.

        Example: [{
          'job_name': 'test',
          'number': 1,
          'url': 'http://localhost:8080/job/test/1/'
        }]
        """
        name = self._normalize_name(name)
        response = await self.jenkins._request(
            'GET',
            '/computer/{}/rssAll'.format(name),
        )
        return _parse_rss(await response.text())

    async def get_config(self, name: str) -> str:
        """
        Return node config in XML format
        """
        name = self._normalize_name(name)
        response = await self.jenkins._request(
            'GET',
            '/computer/{}/config.xml'.format(name)
        )

        return await response.text()

    async def is_exists(self, name: str) -> bool:
        if name == '':
            return False

        try:
            await self.get_info(name)
        except JenkinsNotFoundError:
            return False
        return True

    def construct(self, **kwargs: Any):
        """
        Jenkins node constructor, returns dict to be passed to create method.
        """
        return construct_node_config(**kwargs)

    async def create(self, name: str, config: dict) -> None:
        if name in await self.get_all():
            raise JenkinsError('Node `{}` is already exists'.format(name))

        if 'type' not in config:
            config['type'] = 'hudson.slaves.DumbSlave'

        config['name'] = name

        params = {
            'name': name,
            'type': config['type'],
            'json': json.dumps(config)
        }

        await self.jenkins._request(
            'POST',
            '/computer/doCreateItem',
            params=params,
        )

    async def reconfigure(self, name: str, config: str) -> None:
        name = self._normalize_name(name)

        if name == '(master)':
            raise JenkinsError('Cannot reconfigure master node')

        await self.jenkins._request(
            'POST',
            '/computer/{}/config.xml'.format(name),
            data=config,
            headers={'Content-Type': 'text/xml'},
        )

    async def delete(self, name: str) -> None:
        name = self._normalize_name(name)
        await self.jenkins._request(
            'POST',
            '/computer/{}/doDelete'.format(name)
        )

    async def enable(self, name: str) -> None:
        info = await self.get_info(name)
        if not info['offline']:
            return

        name = self._normalize_name(name)
        await self.jenkins._request(
            'POST',
            '/computer/{}/toggleOffline'.format(name)
        )

    async def disable(self, name: str, message: str = '') -> None:
        info = await self.get_info(name)
        if info['offline']:
            return

        name = self._normalize_name(name)
        await self.jenkins._request(
            'POST',
            '/computer/{}/toggleOffline'.format(name),
            params={'offlineMessage': message}
        )

    async def update_offline_reason(self, name: str, message: str) -> None:
        name = self._normalize_name(name)
        await self.jenkins._request(
            'POST',
            '/computer/{}/changeOfflineCause'.format(name),
            params={'offlineMessage': message}
        )



from .exceptions import JenkinsError as JenkinsError
from typing import Any, List, Tuple

JOB_BUILD_URL_RE: Any

def construct_job_config(*, description: str=..., parameters: List[dict]=..., commands: List[str]=...) -> str: ...
def construct_node_config(name: str, *, remote_fs: str=..., executors: int=...) -> dict: ...
def parse_build_url(build_url: str) -> Tuple[str, int]: ...



from typing import Any

from .exceptions import JenkinsNotFoundError
from .utils import construct_job_config


class Jobs:

    def __init__(self, jenkins):
        self.jenkins = jenkins

    async def get_all(self) -> dict:
        """
        Get list of builds for specified job name. Returned example:

        jobs = [
            {'name': 'test', 'url': 'http://localhost/job/test/'},
            {'name': 'test_builds', 'url': 'http://localhost/job/test_builds/'}
        ]
        """
        status = await self.jenkins.get_status()
        return {v['name']: v for v in status['jobs']}

    async def get_info(self, name: str) -> dict:
        response = await self.jenkins._request(
            'GET',
            '/job/{}/api/json'.format(name)
        )

        return await response.json()

    async def get_config(self, name: str) -> str:
        response = await self.jenkins._request(
            'GET',
            '/job/{}/config.xml'.format(name)
        )

        return await response.text()

    async def is_exists(self, name: str) -> bool:
        try:
            await self.get_info(name)
        except JenkinsNotFoundError:
            return False
        else:
            return True

    def construct_config(self, **kwargs: Any) -> str:
        """
        Jenkins job XML constructor
        """
        return construct_job_config(**kwargs)

    async def create(self, name: str, config: str) -> None:
        headers = {'Content-Type': 'text/xml'}
        params = {'name': name}

        await self.jenkins._request(
            'POST',
            '/createItem',
            params=params,
            data=config,
            headers=headers
        )

    async def reconfigure(self, name: str, config: str) -> None:
        await self.jenkins._request(
            'POST',
            '/job/{}/config.xml'.format(name),
            data=config,
            headers={'Content-Type': 'text/xml'},
        )

    async def delete(self, name: str) -> None:
        await self.jenkins._request(
            'POST',
            '/job/{}/doDelete'.format(name)
        )

    async def copy(self, name: str, new_name: str) -> None:
        params = {
            'mode': 'copy',
            'from': name,
            'name': new_name,
        }

        await self.jenkins._request(
            'POST',
            '/createItem',
            params=params
        )

    async def rename(self, name: str, new_name: str) -> None:
        params = {'newName': new_name}

        await self.jenkins._request(
            'POST',
            '/job/{}/doRename'.format(name),
            params=params
        )

    async def enable(self, name: str) -> None:
        await self.jenkins._request(
            'POST',
            '/job/{}/enable'.format(name)
        )

    async def disable(self, name: str) -> None:
        await self.jenkins._request(
            'POST',
            '/job/{}/disable'.format(name)
        )
